package javax.swing.plaf.metal;

import java.beans.PropertyChangeEvent;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.beans.PropertyChangeListener;
import javax.swing.JSlider;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.Icon;
import java.awt.Color;
import javax.swing.plaf.basic.BasicSliderUI;

public class MetalSliderUI extends BasicSliderUI
{
    protected final int TICK_BUFFER = 4;
    protected boolean filledSlider;
    protected static Color thumbColor;
    protected static Color highlightColor;
    protected static Color darkShadowColor;
    protected static int trackWidth;
    protected static int tickLength;
    private int safeLength;
    protected static Icon horizThumbIcon;
    protected static Icon vertThumbIcon;
    private static Icon SAFE_HORIZ_THUMB_ICON;
    private static Icon SAFE_VERT_THUMB_ICON;
    protected final String SLIDER_FILL = "JSlider.isFilled";
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalSliderUI();
    }
    
    public MetalSliderUI() {
        super(null);
        this.filledSlider = false;
    }
    
    private static Icon getHorizThumbIcon() {
        if (System.getSecurityManager() != null) {
            return MetalSliderUI.SAFE_HORIZ_THUMB_ICON;
        }
        return MetalSliderUI.horizThumbIcon;
    }
    
    private static Icon getVertThumbIcon() {
        if (System.getSecurityManager() != null) {
            return MetalSliderUI.SAFE_VERT_THUMB_ICON;
        }
        return MetalSliderUI.vertThumbIcon;
    }
    
    @Override
    public void installUI(JComponent c) {
        MetalSliderUI.trackWidth = (int)UIManager.get("Slider.trackWidth");
        int intValue = (int)UIManager.get("Slider.majorTickLength");
        this.safeLength = intValue;
        MetalSliderUI.tickLength = intValue;
        MetalSliderUI.horizThumbIcon = (MetalSliderUI.SAFE_HORIZ_THUMB_ICON = UIManager.getIcon("Slider.horizontalThumbIcon"));
        MetalSliderUI.vertThumbIcon = (MetalSliderUI.SAFE_VERT_THUMB_ICON = UIManager.getIcon("Slider.verticalThumbIcon"));
        super.installUI(c);
        MetalSliderUI.thumbColor = UIManager.getColor("Slider.thumb");
        MetalSliderUI.highlightColor = UIManager.getColor("Slider.highlight");
        MetalSliderUI.darkShadowColor = UIManager.getColor("Slider.darkShadow");
        this.scrollListener.setScrollByBlock(false);
        this.prepareFilledSliderField();
    }
    
    @Override
    protected PropertyChangeListener createPropertyChangeListener(JSlider slider) {
        return new MetalSliderUI.MetalPropertyListener();
    }
    
    private void prepareFilledSliderField() {
        this.filledSlider = MetalLookAndFeel.usingOcean();
        Object clientProperty = this.slider.getClientProperty("JSlider.isFilled");
        if (clientProperty != null) {
            this.filledSlider = (boolean)clientProperty;
        }
    }
    
    @Override
    public void paintThumb(Graphics graphics) {
        Rectangle thumbRect = this.thumbRect;
        graphics.translate(thumbRect.x, thumbRect.y);
        if (this.slider.getOrientation() == 0) {
            getHorizThumbIcon().paintIcon(this.slider, graphics, 0, 0);
        }
        else {
            getVertThumbIcon().paintIcon(this.slider, graphics, 0, 0);
        }
        graphics.translate(-thumbRect.x, -thumbRect.y);
    }
    
    private Rectangle getPaintTrackRect() {
        int thumbOverhang = 0;
        int n = 0;
        int n2;
        int n3;
        if (this.slider.getOrientation() == 0) {
            n3 = this.trackRect.height - 1 - this.getThumbOverhang();
            n = n3 - (this.getTrackWidth() - 1);
            n2 = this.trackRect.width - 1;
        }
        else {
            if (MetalUtils.isLeftToRight(this.slider)) {
                thumbOverhang = this.trackRect.width - this.getThumbOverhang() - this.getTrackWidth();
                n2 = this.trackRect.width - this.getThumbOverhang() - 1;
            }
            else {
                thumbOverhang = this.getThumbOverhang();
                n2 = this.getThumbOverhang() + this.getTrackWidth() - 1;
            }
            n3 = this.trackRect.height - 1;
        }
        return new Rectangle(this.trackRect.x + thumbOverhang, this.trackRect.y + n, n2 - thumbOverhang, n3 - n);
    }
    
    @Override
    public void paintTrack(Graphics graphics) {
        if (MetalLookAndFeel.usingOcean()) {
            this.oceanPaintTrack(graphics);
            return;
        }
        Color color = this.slider.isEnabled() ? this.slider.getForeground() : MetalLookAndFeel.getControlShadow();
        int leftToRight = MetalUtils.isLeftToRight(this.slider);
        graphics.translate(this.trackRect.x, this.trackRect.y);
        int thumbOverhang = 0;
        int n = 0;
        int n2;
        int n3;
        if (this.slider.getOrientation() == 0) {
            n3 = this.trackRect.height - 1 - this.getThumbOverhang();
            n = n3 - (this.getTrackWidth() - 1);
            n2 = this.trackRect.width - 1;
        }
        else {
            if (!(leftToRight == 0)) {
                thumbOverhang = this.trackRect.width - this.getThumbOverhang() - this.getTrackWidth();
                n2 = this.trackRect.width - this.getThumbOverhang() - 1;
            }
            else {
                thumbOverhang = this.getThumbOverhang();
                n2 = this.getThumbOverhang() + this.getTrackWidth() - 1;
            }
            n3 = this.trackRect.height - 1;
        }
        if (this.slider.isEnabled()) {
            graphics.setColor(MetalLookAndFeel.getControlDarkShadow());
            graphics.drawRect(thumbOverhang, n, n2 - thumbOverhang - 1, n3 - n - 1);
            graphics.setColor(MetalLookAndFeel.getControlHighlight());
            graphics.drawLine(thumbOverhang + 1, n3, n2, n3);
            graphics.drawLine(n2, n + 1, n2, n3);
            graphics.setColor(MetalLookAndFeel.getControlShadow());
            graphics.drawLine(thumbOverhang + 1, n + 1, n2 - 2, n + 1);
            graphics.drawLine(thumbOverhang + 1, n + 1, thumbOverhang + 1, n3 - 2);
        }
        else {
            graphics.setColor(MetalLookAndFeel.getControlShadow());
            graphics.drawRect(thumbOverhang, n, n2 - thumbOverhang - 1, n3 - n - 1);
        }
        if (this.filledSlider) {
            int n5;
            int n6;
            int n7;
            int n8;
            if (this.slider.getOrientation() == 0) {
                n6 = this.thumbRect.x + this.thumbRect.width / 2 - this.trackRect.x;
                n7 = (this.slider.isEnabled() ? (n + 1) : n);
                n8 = (this.slider.isEnabled() ? (n3 - 2) : (n3 - 1));
                if (!this.drawInverted()) {
                    n5 = (this.slider.isEnabled() ? (thumbOverhang + 1) : thumbOverhang);
                }
                else {
                    n5 = n6;
                    n6 = (this.slider.isEnabled() ? (n2 - 2) : (n2 - 1));
                }
            }
            else {
                int n4 = this.thumbRect.y + this.thumbRect.height / 2 - this.trackRect.y;
                n5 = (this.slider.isEnabled() ? (thumbOverhang + 1) : thumbOverhang);
                n6 = (this.slider.isEnabled() ? (n2 - 2) : (n2 - 1));
                if (!this.drawInverted()) {
                    n7 = n4;
                    n8 = (this.slider.isEnabled() ? (n3 - 2) : (n3 - 1));
                }
                else {
                    n7 = (this.slider.isEnabled() ? (n + 1) : n);
                    n8 = n4;
                }
            }
            if (this.slider.isEnabled()) {
                graphics.setColor(this.slider.getBackground());
                graphics.drawLine(n5, n7, n6, n7);
                graphics.drawLine(n5, n7, n5, n8);
                graphics.setColor(MetalLookAndFeel.getControlShadow());
                graphics.fillRect(n5 + 1, n7 + 1, n6 - n5, n8 - n7);
            }
            else {
                graphics.setColor(MetalLookAndFeel.getControlShadow());
                graphics.fillRect(n5, n7, n6 - n5, n8 - n7);
            }
        }
        graphics.translate(-this.trackRect.x, -this.trackRect.y);
        return;
    }
    
    private void oceanPaintTrack(Graphics graphics) {
        int leftToRight = MetalUtils.isLeftToRight(this.slider);
        int drawInverted = this.drawInverted();
        Color color = (Color)UIManager.get("Slider.altTrackColor");
        Rectangle paintTrackRect = this.getPaintTrackRect();
        graphics.translate(paintTrackRect.x, paintTrackRect.y);
        int width = paintTrackRect.width;
        int height = paintTrackRect.height;
        if (this.slider.getOrientation() == 0) {
            int n3 = this.thumbRect.x + this.thumbRect.width / 2 - paintTrackRect.x;
            if (this.slider.isEnabled()) {
                if (n3 > 0) {
                    graphics.setColor(drawInverted ? MetalLookAndFeel.getControlDarkShadow() : MetalLookAndFeel.getPrimaryControlDarkShadow());
                    graphics.drawRect(0, 0, n3 - 1, height - 1);
                }
                if (n3 < width) {
                    graphics.setColor(drawInverted ? MetalLookAndFeel.getPrimaryControlDarkShadow() : MetalLookAndFeel.getControlDarkShadow());
                    graphics.drawRect(n3, 0, width - n3 - 1, height - 1);
                }
                if (this.filledSlider) {
                    graphics.setColor(MetalLookAndFeel.getPrimaryControlShadow());
                    int n4;
                    if (!(drawInverted == 0)) {
                        n4 = width - 2;
                        graphics.drawLine(1, 1, n3, 1);
                    }
                    else {
                        n3 = (n4 = 1);
                        graphics.drawLine(n3, 1, width - 1, 1);
                    }
                    if (height == 6) {
                        graphics.setColor(MetalLookAndFeel.getWhite());
                        graphics.drawLine(n3, 1, n4, 1);
                        graphics.setColor(color);
                        graphics.drawLine(n3, 2, n4, 2);
                        graphics.setColor(MetalLookAndFeel.getControlShadow());
                        graphics.drawLine(n3, 3, n4, 3);
                        graphics.setColor(MetalLookAndFeel.getPrimaryControlShadow());
                        graphics.drawLine(n3, 4, n4, 4);
                    }
                }
            }
            else {
                graphics.setColor(MetalLookAndFeel.getControlShadow());
                if (n3 > 0) {
                    if (drawInverted == 0 && this.filledSlider) {
                        graphics.fillRect(0, 0, n3 - 1, height - 1);
                    }
                    else {
                        graphics.drawRect(0, 0, n3 - 1, height - 1);
                    }
                }
                if (n3 < width) {
                    if (drawInverted && this.filledSlider) {
                        graphics.fillRect(n3, 0, width - n3 - 1, height - 1);
                    }
                    else {
                        graphics.drawRect(n3, 0, width - n3 - 1, height - 1);
                    }
                }
            }
        }
        else {
            int n = this.thumbRect.y + this.thumbRect.height / 2 - paintTrackRect.y;
            if (this.slider.isEnabled()) {
                if (n > 0) {
                    graphics.setColor(drawInverted ? MetalLookAndFeel.getPrimaryControlDarkShadow() : MetalLookAndFeel.getControlDarkShadow());
                    graphics.drawRect(0, 0, width - 1, n - 1);
                }
                if (n < height) {
                    graphics.setColor(drawInverted ? MetalLookAndFeel.getControlDarkShadow() : MetalLookAndFeel.getPrimaryControlDarkShadow());
                    graphics.drawRect(0, n, width - 1, height - n - 1);
                }
                if (this.filledSlider) {
                    graphics.setColor(MetalLookAndFeel.getPrimaryControlShadow());
                    int n2;
                    if (this.drawInverted()) {
                        n2 = 1;
                        if (!(leftToRight == 0)) {
                            graphics.drawLine(1, n, 1, height - 1);
                        }
                        else {
                            graphics.drawLine(width - 2, n, width - 2, height - 1);
                        }
                    }
                    else {
                        n2 = n;
                        n = height - 2;
                        if (!(leftToRight == 0)) {
                            graphics.drawLine(1, 1, 1, n);
                        }
                        else {
                            graphics.drawLine(width - 2, 1, width - 2, n);
                        }
                    }
                    if (width == 6) {
                        graphics.setColor(leftToRight ? MetalLookAndFeel.getWhite() : MetalLookAndFeel.getPrimaryControlShadow());
                        graphics.drawLine(1, n2, 1, n);
                        graphics.setColor(leftToRight ? color : MetalLookAndFeel.getControlShadow());
                        graphics.drawLine(2, n2, 2, n);
                        graphics.setColor(leftToRight ? MetalLookAndFeel.getControlShadow() : color);
                        graphics.drawLine(3, n2, 3, n);
                        graphics.setColor(leftToRight ? MetalLookAndFeel.getPrimaryControlShadow() : MetalLookAndFeel.getWhite());
                        graphics.drawLine(4, n2, 4, n);
                    }
                }
            }
            else {
                graphics.setColor(MetalLookAndFeel.getControlShadow());
                if (n > 0) {
                    if (drawInverted && this.filledSlider) {
                        graphics.fillRect(0, 0, width - 1, n - 1);
                    }
                    else {
                        graphics.drawRect(0, 0, width - 1, n - 1);
                    }
                }
                if (n < height) {
                    if (drawInverted == 0 && this.filledSlider) {
                        graphics.fillRect(0, n, width - 1, height - n - 1);
                    }
                    else {
                        graphics.drawRect(0, n, width - 1, height - n - 1);
                    }
                }
            }
        }
        graphics.translate(-paintTrackRect.x, -paintTrackRect.y);
    }
    
    @Override
    public void paintFocus(Graphics graphics) {
    }
    
    @Override
    protected Dimension getThumbSize() {
        Dimension dimension = new Dimension();
        if (this.slider.getOrientation() == 1) {
            dimension.width = getVertThumbIcon().getIconWidth();
            dimension.height = getVertThumbIcon().getIconHeight();
            return dimension;
        }
        dimension.width = getHorizThumbIcon().getIconWidth();
        dimension.height = getHorizThumbIcon().getIconHeight();
        return dimension;
    }
    
    public int getTickLength() {
        return (this.slider.getOrientation() == 0) ? (this.safeLength + 4 + 1) : (this.safeLength + 4 + 3);
    }
    
    protected int getTrackWidth() {
        if (this.slider.getOrientation() == 0) {
            return (int)(0.4375 * (double)this.thumbRect.height);
        }
        return (int)(0.4375 * (double)this.thumbRect.width);
    }
    
    protected int getTrackLength() {
        if (this.slider.getOrientation() == 0) {
            return this.trackRect.width;
        }
        return this.trackRect.height;
    }
    
    protected int getThumbOverhang() {
        return (int)(this.getThumbSize().getHeight() - (double)this.getTrackWidth()) / 2;
    }
    
    @Override
    protected void scrollDueToClickInTrack(int direction) {
        this.scrollByUnit(direction);
    }
    
    @Override
    protected void paintMinorTickForHorizSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.setColor(this.slider.isEnabled() ? this.slider.getForeground() : MetalLookAndFeel.getControlShadow());
        graphics.drawLine(n, 4, n, 4 + this.safeLength / 2);
    }
    
    @Override
    protected void paintMajorTickForHorizSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.setColor(this.slider.isEnabled() ? this.slider.getForeground() : MetalLookAndFeel.getControlShadow());
        graphics.drawLine(n, 4, n, 4 + (this.safeLength - 1));
    }
    
    @Override
    protected void paintMinorTickForVertSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.setColor(this.slider.isEnabled() ? this.slider.getForeground() : MetalLookAndFeel.getControlShadow());
        if (MetalUtils.isLeftToRight(this.slider)) {
            graphics.drawLine(4, n, 4 + this.safeLength / 2, n);
        }
        else {
            graphics.drawLine(0, n, this.safeLength / 2, n);
        }
    }
    
    @Override
    protected void paintMajorTickForVertSlider(Graphics graphics, Rectangle rectangle, int n) {
        graphics.setColor(this.slider.isEnabled() ? this.slider.getForeground() : MetalLookAndFeel.getControlShadow());
        if (MetalUtils.isLeftToRight(this.slider)) {
            graphics.drawLine(4, n, 4 + this.safeLength, n);
        }
        else {
            graphics.drawLine(0, n, this.safeLength, n);
        }
    }
    
    static void access$000(MetalSliderUI metalSliderUI) {
        metalSliderUI.prepareFilledSliderField();
    }
    
    protected class MetalPropertyListener extends BasicSliderUI.PropertyChangeHandler
    {
        @Override
        public void propertyChange(PropertyChangeEvent e) {
            super.propertyChange(e);
            if (e.getPropertyName().equals("JSlider.isFilled")) {
                MetalSliderUI.access$000(MetalSliderUI.this);
            }
        }
    }
}
